@@ -8,7 +8,7 @@
 #include	"authsasl.h"
 #include	<stdio.h>
 
-static const char rcsid[]="$Id: authinfo.c,v 1.3 1999/12/13 03:34:04 mrsam Exp $";
+static const char rcsid[]="$Id: authinfo.c,v 1.4 2000/04/26 22:32:37 mrsam Exp $";
 
 int main()
 {
@@ -18,6 +18,10 @@ int	i;
 	for (i=0; authstaticlist[i].auth_name; i++)
 		printf("%s%s", i ? " ":"", authstaticlist[i].auth_name);
 	printf("\"\n");
+#ifdef	AUTHDAEMONMODULELIST
+	printf("AUTHDAEMONMODULELIST=\"%s\"\n",
+		AUTHDAEMONMODULELIST);
+#endif
 	printf("SASL_AUTHENTICATION_MODULES=\"");
 	for (i=0; authsasl_list[i].sasl_method; i++)
 		printf("%s%s", i ? " ":"", authsasl_list[i].sasl_method);
