@@ -146,9 +146,9 @@ static int read_env(const char *env, const char **copy,
 	const char *err, int needit, const char *value_default)
 {
 static char *ldapauth=0;
-static size_t ldapauth_size;
+static size_t ldapauth_size=0;
 size_t	i;
-char	*p;
+char	*p=0;
 int	l=strlen(env);
 
 	if (!ldapauth)
