@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -26,7 +26,7 @@
 #include	<sys/types.h>
 #include	<sys/stat.h>
 
-static const char rcsid[]="$Id: fetch.c,v 1.7 2000/04/30 15:19:22 mrsam Exp $";
+static const char rcsid[]="$Id: fetch.c,v 1.9 2000/08/20 17:22:35 mrsam Exp $";
 
 extern int current_mailbox_ro;
 extern struct imapscaninfo current_mailbox_info;
@@ -208,8 +208,7 @@ char	quotabuf[QUOTABUFSIZE];
 	mi->filename=p;
 
 #if 0
-	if (strncmp(current_mailbox, SHAREDSUBDIR "/",
-		sizeof(SHAREDSUBDIR "/")) == 0)
+	if (is_sharedsubdir(current_mailbox))
 		maildir_shared_updateflags(current_mailbox, p);
 #endif
 
@@ -585,6 +584,9 @@ off_t start_pos, end_pos, start_body;
 off_t nlines, nbodylines;
 unsigned long cnt;
 char	buf[BUFSIZ];
+char	rbuf[BUFSIZ];
+char	*rbufptr;
+int	rbufleft;
 unsigned bufptr;
 unsigned long skipping;
 int	ismsgrfc822=1;
@@ -730,16 +732,29 @@ int	ismsgrfc822=1;
 	bufptr=0;
 	writeflush();
 
+	rbufptr=0;
+	rbufleft=0;
+
 	while (cnt)
 	{
-	int	c=getc(fp);
+	int	c;
 
-		if (c == EOF)
+		if (!rbufleft)
+		{
+			rbufleft=fread(rbuf, 1, sizeof(rbuf), fp);
+			if (rbufleft < 0)	rbufleft=0;
+			rbufptr=rbuf;
+		}
+
+		if (!rbufleft)
 		{
 			fetcherror("unexpected EOF", fi, i, msgnum);
 			_exit(1);
 		}
 
+		--rbufleft;
+		c=(int)(unsigned char)*rbufptr++;
+
 		if (c == '\n')
 		{
 			if (skipping)
@@ -834,12 +849,12 @@ static void dofetchheadersbuf(FILE *fp, struct fetchinfo *fi,
 off_t start_pos, end_pos, start_body;
 off_t nlines, nbodylines;
 size_t i,j,k,l;
-int	c;
 char	buf[BUFSIZ+2];
 int	goodheader;
 unsigned long skipping;
 unsigned long cnt;
 char	*p;
+int	ii;
 
 	rfc2045_mimepos(mimep, &start_pos, &end_pos, &start_body,
 		&nlines, &nbodylines);
@@ -849,15 +864,12 @@ char	*p;
 		fetcherror("fseek", fi, info, msgnum);
 		return;
 	}
-	for (i=0; i<start_body-start_pos; i++)
+
+	ii=fread(buf, 1, start_body - start_pos, fp);
+	if (ii <= 0 || (i=ii) != start_body - start_pos)
 	{
-		c=getc(fp);
-		if (c == EOF)
-		{
-			fetcherror("unexpected EOF", fi, info, msgnum);
-			_exit(1);
-		}
-		buf[i]=c;
+		fetcherror("unexpected EOF", fi, info, msgnum);
+		exit(1);
 	}
 	goodheader= (*headerfunc)(fi, "");
 
