@@ -68,9 +68,11 @@
 #include	"maildir/maildirquota.h"
 #include	"maildir/maildirmisc.h"
 
+#include	"unicode/unicode.h"
+
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.44 2000/09/02 21:52:25 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.47 2000/09/29 04:38:47 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -975,14 +977,11 @@ int	move_to_trash=0;
 				(p=strchr(p, 'T')))
 				while ((*p=p[1]) != 0)
 					++p;
-			if (rename(old_name, new_name))
-			{
-				writes("* NO [ALERT] Error while moving ");
-				writes(old_name);
-				writes(" to ");
-				writes(new_name);
-				writes("\r\n");
-			}
+			rename(old_name, new_name);
+			unlink(old_name);
+			/* triggers linux kernel bug if also moved to Trash by
+			sqwebmail */
+
 			free(new_name);
 		}
 		else
@@ -1975,6 +1974,7 @@ int	uid=0;
 	if (strcmp(curtoken->tokenbuf, "SEARCH") == 0)
 	{
 	char *charset=0;
+	const struct unicode_info *mycharset;
 	struct searchinfo *si, *sihead;
 	unsigned long i;
 
@@ -2001,26 +2001,29 @@ int	uid=0;
 			free_search(sihead);
 			return (-1);
 		}
-		if (charset && strcasecmp(charset, "US-ASCII") == 0)
-		{
-			free(charset);
-			charset=0;
-		}
-		if (charset)
+
+		mycharset=unicode_find(charset);
+
+		if (!mycharset)
 		{
 			writes(tag);
-			writes(" NO Can't search that character set.\r\n");
+			writes(" NO ");
+			writes(charset);
+			writes(" character set is not supported.\r\n");
 			if (charset)	free(charset);
 			free_search(sihead);
 			return (0);
 		}
+
+		if (charset)
+			free(charset);
 #if 0
 		writes("* OK ");
 		debug_search(si);
 		writes("\r\n");
 #endif
 		writes("* SEARCH");
-		dosearch(si, sihead, uid);
+		dosearch(si, sihead, mycharset, uid);
 		writes("\r\n");
 
 		for (i=0; i<current_mailbox_info.nmessages; i++)
@@ -2037,10 +2040,12 @@ int	uid=0;
 	char *charset=0;
 	struct searchinfo *si, *sihead;
 	unsigned long i;
+	const struct unicode_info *mycharset;
 
 		/* The following jazz is mainly for future extensions */
 
-	void (*thread_func)(struct searchinfo *, struct searchinfo *, int);
+	void (*thread_func)(struct searchinfo *, struct searchinfo *,
+			    const struct unicode_info *, int);
 	search_type thread_type;
 
 		{
@@ -2080,7 +2085,6 @@ int	uid=0;
 			curtoken->tokentype != IT_QUOTED_STRING)
 			return (-1);
 
-
 		charset=my_strdup(curtoken->tokenbuf);
 		curtoken=nexttoken();
 
@@ -2098,22 +2102,23 @@ int	uid=0;
 			free_search(sihead);
 			return (-1);
 		}
-		if (charset && strcasecmp(charset, "US-ASCII") == 0)
-		{
-			free(charset);
-			charset=0;
-		}
-		if (charset)
+
+		mycharset=unicode_find(charset);
+		if (!mycharset)
 		{
 			writes(tag);
-			writes(" NO Can't thread with that character set.\r\n");
-			if (charset)	free(charset);
+			writes(" NO ");
+			writes(charset);
+			writes(" character set is not supported.\r\n");
+
+			free(charset);
 			free_search(sihead);
 			return (0);
 		}
+		free(charset);
 
 		writes("* THREAD ");
-		(*thread_func)(si, sihead, uid);
+		(*thread_func)(si, sihead, mycharset, uid);
 		writes("\r\n");
 
 		for (i=0; i<current_mailbox_info.nmessages; i++)
@@ -2131,6 +2136,7 @@ int	uid=0;
 	struct searchinfo *si, *sihead;
 	unsigned long i;
 	struct temp_sort_stack *ts=0;
+	const struct unicode_info *mycharset;
 
 		{
 		const char *p=getenv("IMAP_DISABLETHREADSORT");
@@ -2245,27 +2251,24 @@ int	uid=0;
 			free_search(sihead);
 			return (-1);
 		}
-		if (charset && strcasecmp(charset, "US-ASCII") == 0)
-		{
-			free(charset);
-			charset=0;
-		}
-		if (charset && strcasecmp(charset, "UTF-8") == 0)
-		{
-			free(charset);
-			charset=0;
-		}
-		if (charset)
+
+		mycharset=unicode_find(charset);
+
+		if (!mycharset)
 		{
 			writes(tag);
-			writes(" NO Can't sort with that character set.\r\n");
-			if (charset)	free(charset);
+			writes(" NO ");
+			writes(charset);
+			writes(" character set is not supported.\r\n");
+
+			if (charset) free(charset);
 			free_search(sihead);
 			return (0);
 		}
+		free(charset);
 
 		writes("* SORT");
-		dosortmsgs(si, sihead, uid);
+		dosortmsgs(si, sihead, mycharset, uid);
 		writes("\r\n");
 
 		for (i=0; i<current_mailbox_info.nmessages; i++)
