@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -13,7 +13,7 @@
 #include	<stdlib.h>
 #include	<string.h>
 
-static const char rcsid[]="$Id: msgenvelope.c,v 1.5 2000/07/30 01:08:09 mrsam Exp $";
+static const char rcsid[]="$Id: msgenvelope.c,v 1.6 2000/09/09 23:07:54 mrsam Exp $";
 
 #define	MAX_HEADER_SIZE	8192
 
@@ -135,7 +135,7 @@ char	*q, *r;
 			a->addrs[i].name->next == 0)
 			a->addrs[i].name->token=0;
 
-		q=rfc822_gettok(a->addrs[i].name);
+		q=rfc822_getname(a, i);
 		if (!q)
 		{
 			perror("malloc");
