@@ -15,18 +15,20 @@
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
+#include	<errno.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	"rfc822/rfc822.h"
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045/rfc2045.h"
+#include	"unicode/unicode.h"
 #include	"numlib/numlib.h"
 #include	"searchinfo.h"
 #include	"imapwrite.h"
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.12 2000/09/02 23:55:55 mrsam Exp $";
+static const char rcsid[]="$Id: search.c,v 1.14 2000/09/29 04:38:47 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern struct imapscaninfo current_mailbox_info;
@@ -42,10 +44,13 @@ static void fill_search_veryquick(struct searchinfo *,
 static void fill_search_quick(struct searchinfo *,
 	unsigned long, struct stat *);
 
-static void fill_search_header(struct searchinfo *, struct rfc2045 *, FILE *,
-	struct imapscanmessageinfo *);
+static void fill_search_header(struct searchinfo *,
+			       const struct unicode_info *,
+			       struct rfc2045 *, FILE *,
+			       struct imapscanmessageinfo *);
 
 static void fill_search_body(struct searchinfo *,
+			     const struct unicode_info *,
 	struct rfc2045 *, FILE *, struct imapscanmessageinfo *);
 
 static int search_evaluate(struct searchinfo *);
@@ -69,9 +74,10 @@ static void search_callback(struct searchinfo *, struct searchinfo *, int,
 **	number.
 */
 
-void dosearch(struct searchinfo *si, struct searchinfo *sihead, int isuid)
+void dosearch(struct searchinfo *si, struct searchinfo *sihead,
+	      const struct unicode_info *mycharset, int isuid)
 {
-	search_internal(si, sihead, isuid, search_callback, 0);
+	search_internal(si, sihead, mycharset, isuid, search_callback, 0);
 }
 
 static void search_callback(struct searchinfo *si, struct searchinfo *sihead,
@@ -82,8 +88,11 @@ static void search_callback(struct searchinfo *si, struct searchinfo *sihead,
 }
 
 void search_internal(struct searchinfo *si, struct searchinfo *sihead,
-	int isuid, void (*callback_func)(struct searchinfo *,
-		struct searchinfo *, int, unsigned long, void *), void *voidarg)
+		     const struct unicode_info *mycharset, int isuid,
+		     void (*callback_func)(struct searchinfo *,
+					   struct searchinfo *, int,
+					   unsigned long, void *),
+		     void *voidarg)
 {
 unsigned long i;
 struct	searchinfo *p;
@@ -137,7 +146,7 @@ int	rc;
 
 			/* No, search the headers then */
 
-			fill_search_header(sihead, rfcp, fp,
+			fill_search_header(sihead, mycharset, rfcp, fp,
 				current_mailbox_info.msgs+i);
 			rc=search_evaluate(si);
 
@@ -145,7 +154,7 @@ int	rc;
 			{
 				/* Ok, search message contents */
 
-				fill_search_body(si, rfcp, fp,
+				fill_search_body(si, mycharset, rfcp, fp,
 					current_mailbox_info.msgs+i);
 				rc=search_evaluate(si);
 			}
@@ -460,9 +469,11 @@ int	rc, rc2;
 /* ------- header search -------- */
 
 static void fill_search_header_recursive(struct searchinfo *,
+					 const struct unicode_info *,
 	struct rfc2045 *, FILE *, struct imapscanmessageinfo *);
 
 static void fill_search_header(struct searchinfo *si,
+			       const struct unicode_info *mycharset,
 	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
 {
 	off_t start_pos, end_pos, start_body;
@@ -487,17 +498,27 @@ static void fill_search_header(struct searchinfo *si,
 	rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
 		&nlines, &nbodylines);
 
+	/*
+	** Since headers can contain any character set, we set the search
+	** strings to their native character set.
+	*/
+
+	search_set_charset_conv(si, mycharset, mycharset);
+
 	if (fseek(fp, start_pos, SEEK_SET) < 0)
 	{
 		searcherror("fseek failed", mi);
 		return;
 	}
+
 	while (start_pos < start_body)
 	{
 	char	*p=readline(0, fp);
 	char	*q=0;
 	char	*r;
 	int	isdate;
+	char *conv_buf;
+	char *conv_buf_uc;
 
 		/* Collect multiline header into the readline buffer */
 
@@ -518,32 +539,52 @@ static void fill_search_header(struct searchinfo *si,
 
 		/* search_text applies to headers too */
 
+		for (r=p; *r && *r != ':'; r++)
+			*r=toupper(*r);
+
+		if (r)
+			*r++=0;
+
+		while (*r && isspace((int)(unsigned char)*r))
+			++r;
+
+		conv_buf=rfc2047_decode_unicode(r, mycharset,
+						RFC2047_DECODE_DISCARD);
+
+			/* Characters not in our charset are discarded */
+
+		if (!conv_buf && errno != EINVAL)
+			write_error_exit(0);
+
+		conv_buf_uc= (*mycharset->toupper_func)(conv_buf, 0);
+		free(conv_buf);
+		if (!conv_buf_uc)
+			write_error_exit(0);
+
 		for (sip=si; sip; sip=sip->next)
-			if (sip->type == search_text && sip->value <= 0)
+		{
+			if (sip->type == search_text &&
+			    sip->bs &&
+			    sip->value <= 0)
+			{
 				search_reset(sip);
 
-		for (q=p; *q; q++)
-		{
-			*q=toupper((int)(unsigned char)*q);
-			for (sip=si; sip; sip=sip->next)
-				if (sip->type == search_text &&
-					sip->value <= 0)
-					search_step(sip, *q);
+				for (q=conv_buf_uc; *q; q++)
+					search_step(sip, (unsigned char)*q);
+				if (search_found(sip))
+					sip->value=1;
+			}
 		}
 
-		if ((q=strchr(p, ':')) != 0)
-		{
-			*q++=0;
-			while (*q && isspace((int)(unsigned char)*q))
-				++q;
-		}	else q="";
+		q=conv_buf_uc;
 
 		isdate=strcmp(p, "DATE");
 		for (sip=si; sip; sip=sip->next)
 		{
 			if (sip->type == search_text)
 			{
-				if (sip->value <= 0 && search_found(sip))
+				if (sip->value <= 0 &&
+				    sip->bs && search_found(sip))
 					sip->value=1;
 			}
 
@@ -650,9 +691,27 @@ static void fill_search_header(struct searchinfo *si,
 			if (sip->type == search_references1 &&
 			    strcmp(p, "IN-REPLY-TO") == 0 && sip->bs == 0)
 			{
-				sip->bs=strdup(q);
+				/* Extract first address from In-REPLY-To */
+
+				struct rfc822t *t;
+				struct rfc822a *a;
+				char *s;
+
+				t=rfc822t_alloc(q, 0);
+				if (!t) write_error_exit(0);
+				a=rfc822a_alloc(t);
+				if (!a) write_error_exit(0);
+				s=a->naddrs > 0 ? rfc822_getaddr(a, 0):
+					strdup("");
+				rfc822a_free(a);
+				rfc822t_free(t);
+				if (!s) write_error_exit(0);
+
+				sip->bs=malloc(strlen(s)+3);
 				if (!sip->bs)
 					write_error_exit(0);
+				strcat(strcat(strcpy(sip->bs, "<"), s), ">");
+				free(s);
 			}
 
 			if (sip->type == search_references2 &&
@@ -688,7 +747,7 @@ static void fill_search_header(struct searchinfo *si,
 			/* Dummy node used to suck in the Subject:
 			** header for the THREAD or SORT command.
 			*/
-				r=rfc2047_decode_simple(q);
+				r=rfc2047_decode_unicode(q, &unicode_UTF8, 0);
 				if (!r)
 					write_error_exit(0);
 
@@ -699,9 +758,10 @@ static void fill_search_header(struct searchinfo *si,
 			}
 
 			if (sip->type != search_header ||
+			    sip->bs == 0 ||
 				sip->value > 0)	continue;
 
-			if (strcmp(sip->as, p) == 0)
+			if (strcmp(sip->cs, p) == 0)
 			{
 				sip->value=0;
 				search_reset(sip);
@@ -711,13 +771,15 @@ static void fill_search_header(struct searchinfo *si,
 					sip->value=1;
 			}
 		}
+		free(conv_buf_uc);
 	}
-	fill_search_header_recursive(si, rfcp, fp, mi);
+	fill_search_header_recursive(si, mycharset, rfcp, fp, mi);
 }
 
 /* Scan the headers of attached messages too! */
 
 static void fill_search_header_recursive(struct searchinfo *si,
+					 const struct unicode_info *mycharset,
 	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
 {
 	if (!rfcp)	return;
@@ -725,7 +787,7 @@ static void fill_search_header_recursive(struct searchinfo *si,
 	if (rfcp->firstpart && rfcp->firstpart->next == 0 &&
 		!rfcp->firstpart->isdummy)
 	{
-		fill_search_header(si, rfcp->firstpart, fp, mi);
+		fill_search_header(si, mycharset, rfcp->firstpart, fp, mi);
 		return;
 	}
 
@@ -734,7 +796,8 @@ static void fill_search_header_recursive(struct searchinfo *si,
 	while (rfcp)
 	{
 		if (!rfcp->isdummy)
-			fill_search_header_recursive(si, rfcp, fp, mi);
+			fill_search_header_recursive(si, mycharset,
+						     rfcp, fp, mi);
 		rfcp=rfcp->next;
 	}
 }
@@ -743,7 +806,15 @@ static int search_body_func(const char *, size_t, void *);
 
 /* Last straw - search message contents */
 
+struct search_linebuf {
+	char buf[BUFSIZ];
+	int l;
+	const struct unicode_info *uc;
+	struct searchinfo *si;
+} ;
+
 static void fill_search_body(struct searchinfo *si,
+			     const struct unicode_info *mycharset,
 	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
 {
 struct searchinfo *sip;
@@ -755,6 +826,8 @@ const char *charset_s;
 char	buf[BUFSIZ];
 int	rc;
 
+ struct search_linebuf slb;
+
 	for (sip=si; sip; sip=sip->next)
 		if ((sip->type == search_text || sip->type == search_body)
 			&& sip->value <= 0)
@@ -767,28 +840,35 @@ int	rc;
 	if (rfcp->firstpart)	/* Search multiline contents */
 	{
 		for (rfcp=rfcp->firstpart; rfcp; rfcp=rfcp->next)
-			fill_search_body(si, rfcp, fp, mi);
+			fill_search_body(si, mycharset, rfcp, fp, mi);
 		return;
 	}
 
-	while (sip)
-	{
-		if ((sip->type == search_text || sip->type == search_body)
-			&& sip->value <= 0)
-			search_reset(sip);
-		sip=sip->next;
-	}
+	/*
+	** Since headers can contain any character set, we set the search
+	** strings to their native character set.
+	*/
+
 	rfc2045_mimeinfo(rfcp, &content_type_s,
 		&content_transfer_encoding_s, &charset_s);
 	rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
 		&nlines, &nbodylines);
 
+	slb.l=0;
+	slb.uc=unicode_find(charset_s);
+	slb.si=si;
+
+	if (!slb.uc)
+		return;	/* Don't know how to search this charset */
+
+	search_set_charset_conv(si, mycharset, slb.uc);
+
 	if (fseek(fp, start_body, SEEK_SET) < 0)
 	{
 		searcherror("fseek failed", mi);
 		return;
 	}
-	rfc2045_cdecode_start(rfcp, search_body_func, si);
+	rfc2045_cdecode_start(rfcp, search_body_func, &slb);
 
 	rc=0;
 	while (start_body < end_pos)
@@ -812,7 +892,8 @@ int	rc;
 	if (rc == 0)	(void)rfc2045_cdecode_end(rfcp);
 	for (sip=si; sip; sip=sip->next)
 		if ((sip->type == search_text || sip->type == search_body)
-			&& sip->value <= 0)
+		    && sip->bs
+		    && sip->value <= 0)
 		{
 			if (search_found(sip))
 				sip->value=1;
@@ -823,46 +904,49 @@ int	rc;
 
 static int search_body_func(const char *ptr, size_t cnt, void *voidptr)
 {
-struct searchinfo *si=(struct searchinfo *)voidptr;
-size_t	i;
+	struct search_linebuf *slb=(struct search_linebuf *)voidptr;
 
-	while (si)
+	struct searchinfo *si, *sihead=(struct searchinfo *)slb->si;
+	size_t	i, j;
+	int rc=1;
+
+	for (i=0; i<cnt; i++)
 	{
-		if ((si->type == search_text || si->type == search_body)
-			&& si->value <= 0)
-			break;
-		si=si->next;
-	}
-	if (si == 0)	return (1);
-	do
-	{
-		for (i=0; i<cnt; i++)
+		char c=ptr[i];
+		char *p;
+
+		if (c != '\n' && c != '\r')
 		{
-		unsigned char c=ptr[i];
+			if (slb->l < sizeof(slb->buf)-2)
+				slb->buf[slb->l++]=c;
+			continue;
+		}
+		slb->buf[slb->l]=0;
 
-			if (c == '\n')
+		p=(*slb->uc->toupper_func)(slb->buf, 0);
+		if (!p)
+			write_error_exit(0);
+
+		for (si=sihead; si; si=si->next)
+		{
+			if ((si->type == search_text ||
+			     si->type == search_body)
+			    && si->bs && si->value <= 0)
 			{
-				if (search_found(si))
-				{
-					si->value=1;
-					break;
-				}
 				si->value=0;
 				search_reset(si);
-			}
-			else
-			{
-				c=toupper((int)c);
-				search_step(si, c);
-			}
-		}
 
-		while ( (si=si->next) != 0)
-		{
-			if ((si->type == search_text || si->type == search_body)
-				&& si->value <= 0)
-				break;
+				for (j=0; p[j]; j++)
+					search_step(si, (unsigned char)p[j]);
+
+				if (search_found(si))
+					si->value=1;
+				else
+					rc=0;
+			}
 		}
-	} while (si);
+		free(p);
+		slb->l=0;
+	}
 	return (0);
 }
