@@ -41,7 +41,7 @@
 #include	"maildircreate.h"
 #include	"maildirsharedrc.h"
 
-static const char rcsid[]="$Id: maildirshared.c,v 1.12 2000/05/07 17:14:24 mrsam Exp $";
+static const char rcsid[]="$Id: maildirshared.c,v 1.13 2000/08/02 05:07:44 mrsam Exp $";
 
 /* Prerequisited for shared folder support */
 
@@ -883,14 +883,7 @@ char	*buf=maildir_getlink(filename);
 
 	if (buf)
 	{
-		if (access(buf, 0) == 0)
-		{
-			if (unlink(buf))
-			{
-				free(buf);
-				return (-1);
-			}
-		}
+		unlink(buf);
 		free(buf);
 	}
 	unlink(filename);
